#include <cstdio>
#include <vector>
#include <cstring>
#include <utility>

using namespace std;

const int MAXN = 1 << 10;

int N, M, father[MAXN], sz[MAXN];
vector < pair < int, int > > edges;
int snow[MAXN][MAXN], in[MAXN][MAXN], used[MAXN];
vector < int > v[MAXN];

void scan(){
    scanf ( "%d%d", &N, &M );

    for ( int i = 0; i < M; ++i ){
        int x, y;
        scanf ( "%d%d", &x, &y );
        edges.push_back ( make_pair ( x, y ) );
        in[x][y] = in[y][x] = 1;
        v[x].push_back ( y );
        v[y].push_back ( x );
    }
}

void init(){
    for ( int i = 1; i <= N; ++i ){
        father[i] = i;
        sz[i] = 0;
    }
}


inline int root ( int x ){
    while ( x != father[x] )
        x = father[x];

    return x;
}

void query(){
    int x, y;
    scanf ( "%d%d", &x, &y );

    if ( root ( x ) == root ( y ) )
        printf ( "1" );
    else
        printf ( "0" );
}


inline void merge ( int x, int y ){
    x = root ( x );
    y = root ( y );

    if ( x == y )
        return;

    if ( sz[x] > sz[y] ){
        father[y] = x;
    }
    else if ( sz[x] < sz[y] ){
        father[x] = y;
    }
    else{
        father[x] = y;
        ++sz[y];
    }
}

void add (){
    int x, y;

    scanf ( "%d%d", &x, &y );

    merge ( x, y );

    if ( in[x][y] ){
        snow[x][y] = snow[y][x] = 0;
    }
    else{
        in[x][y] = in[y][x] = 1;
        edges.push_back ( make_pair ( x, y ) );
    }
}

void dfs ( int i ){
    used[i] = 1;

    for ( int j = 0; j < v[i].size(); ++j )
        if ( !used[ v[i][j] ] && !snow[i][ v[i][j] ] ){
            merge ( i, v[i][j] );
            dfs ( v[i][j] );
        }
}

void del (){
    int x, y;
    scanf ( "%d%d",&x, &y );
    snow[x][y] = snow[y][x] = 1;

    memset ( used, 0, sizeof ( used ) );
    init();

    for ( int i = 1; i <= N; ++i )
        if ( !used[i] )
            dfs ( i );
}

void solve(){
    init();
    for ( int i = 0; i < M; ++i )
        merge ( edges[i].first, edges[i].second );

    int Q;

    scanf ( "%d", &Q );

    for ( int i = 0; i < Q; ++i ){
        int x;
        scanf ( "%d", &x );

        if ( x == 1 )
            query();
        else if ( x == 2 )
            add();
        else
            del();
    }
    printf ( "\n" );
}

int main(){
    scan();
    solve();
}
