/*
	Author: Boris Yonchev 
	School: TUES
*/
#include <iostream>
#include <string>
#include <vector>
using namespace std;

unsigned short N, M;

unsigned int Q;

enum CallType { QUESTION, CLEANED, BLOCKED };

struct City
{
    unsigned short id;
    vector<City*> connections;
};

vector<City> cities;

struct Call
{
    CallType type;
    City *city_a, *city_b;
};

vector<Call> calls;

void read_input()
{
    cin >> N >> M;
    
    for (unsigned short i = 0; i < N; i++)
    {
        City city;
        city.id = i+1;
        
        cities.push_back(city);
    }
    
    for (unsigned short i = 0; i < M; i++)
    {
        unsigned short a, b;
        cin >> a >> b;
        
        cities[a-1].connections.push_back(&cities[b-1]);
        cities[b-1].connections.push_back(&cities[a-1]);
    }
    
    cin >> Q;
    
    for (unsigned int i = 0; i < Q; i++)
    {
        unsigned short code, city_a, city_b;
        cin >> code >> city_a >> city_b;
    
        Call call;
        
        call.city_a = &cities[city_a-1];
        call.city_b = &cities[city_b-1];
    
        switch(code)
        {
            case 1:
                call.type = QUESTION;
                break;
            case 2:
                call.type = CLEANED;
                break;
            case 3:
                call.type = BLOCKED;
                break;                
        }
        
        calls.push_back(call);
    }
}

int calc_max_connections()
{
    int result = 0;
    for (int i = 1; i <= N; i++)
    {
        result += i;
    }
    return result*2;
}

bool is_connected(City *a, City *b, int depth)
{    
    if (depth == 0)
        return false;
        
    for(vector<City*>::iterator it = a->connections.begin(); it != a->connections.end(); it++)
    {
        if (*it == b)
        {
            return true;
        }
        else if (is_connected(*it, b, depth-1))
        {
            return true;
        }
    }
    
    return false;
}

bool is_connected(City *a, City *b)
{
    return is_connected(a, b, calc_max_connections() / 10);    
}

string handle_calls()
{
    string result;
    
    for (vector<Call>::iterator it = calls.begin(); it != calls.end(); it++)
    {
        if (it->type == CLEANED)
        {
            it->city_a->connections.push_back(it->city_b);
            it->city_b->connections.push_back(it->city_a);
        }
        else if (it->type == BLOCKED)
        {            
            /* Erasing city_a's connection to city_b */
            for(vector<City*>::iterator it2 = it->city_a->connections.begin(); it2 != it->city_a->connections.end(); it2++)
            {
                if (*it2 == it->city_b)
                {
                    it->city_a->connections.erase(it2);
                    break;
                }
            }
                        
        }
        else if (it->type == QUESTION)
        {
            result.append(is_connected(it->city_a, it->city_b) ? "1" : "0");
        }
    }
    
    return result;
}

int main()
{
    read_input();
    
    cout << handle_calls();
    
    return 0;
}
